// Unreal Fighting Framework by Kevin Poretti

#pragma once

// FF includes
#include "Utils/TCircleBuffer.h"

// UE includes
#include "CoreMinimal.h"
#include "Components/ActorComponent.h"

#include "FFInputBufferComponent.generated.h"

/**
 *	Struct representing the state of a player's inputs for one frame
 */
USTRUCT()
struct FFFInputState
{
    GENERATED_BODY()

	FVector2D	MoveAxes;
	FVector2D	LookAxes;

	UPROPERTY(EditAnywhere, Meta = (Bitmask))
	int32		Buttons;
};

USTRUCT(BlueprintType)
struct FFFInputCondition
{
    GENERATED_BODY()

    UPROPERTY(EditAnywhere)
	TArray<FFFInputState> Sequence;

	UPROPERTY(EditAnywhere)
	int32 Lenience;
};

UCLASS( ClassGroup=(Custom), meta=(BlueprintSpawnableComponent) )
class UNREALFIGHTINGFRAMEWORK_API UFFInputBufferComponent : public UActorComponent
{
	GENERATED_BODY()

public:	
	UFFInputBufferComponent();

	void AddInput(const FFFInputState& InputState);

	bool CheckInputSequence(const FFFInputCondition& InputCondition);

protected:
	/** The underlying buffer data structure for holding past input states */
	TCircleBuffer<FFFInputState, 120> InputBuffer;
};
